@@ -37,25 +37,23 @@ typedef unsigned char _Bool;
 #define _cffi_to_c_double PyFloat_AsDouble
 #define _cffi_to_c_float PyFloat_AsDouble
 
-#define _cffi_from_c_SIGNED(x, type)                                     \
-    (sizeof(type) <= sizeof(long) ? PyInt_FromLong(x) :                  \
-                                    PyLong_FromLongLong(x))
-#define _cffi_from_c_UNSIGNED(x, type)                                   \
-    (sizeof(type) < sizeof(long) ? PyInt_FromLong(x) :                   \
-     sizeof(type) == sizeof(long) ? PyLong_FromUnsignedLong(x) :         \
-                                    PyLong_FromUnsignedLongLong(x))
+#define _cffi_from_c_int(x, type)                                        \
+    (((type)-1) > 0 ?   /* unsigned */                                   \
+        (sizeof(type) < sizeof(long) ? PyInt_FromLong(x) :               \
+         sizeof(type) == sizeof(long) ? PyLong_FromUnsignedLong(x) :     \
+                                        PyLong_FromUnsignedLongLong(x))  \
+      : (sizeof(type) <= sizeof(long) ? PyInt_FromLong(x) :              \
+                                        PyLong_FromLongLong(x)))
 
-#define _cffi_to_c_SIGNED(o, type)                                       \
-    (sizeof(type) == 1 ? _cffi_to_c_i8(o) :                              \
-     sizeof(type) == 2 ? _cffi_to_c_i16(o) :                             \
-     sizeof(type) == 4 ? _cffi_to_c_i32(o) :                             \
-     sizeof(type) == 8 ? _cffi_to_c_i64(o) :                             \
-     (Py_FatalError("unsupported size for type " #type), 0))
-#define _cffi_to_c_UNSIGNED(o, type)                                     \
-    (sizeof(type) == 1 ? _cffi_to_c_u8(o) :                              \
-     sizeof(type) == 2 ? _cffi_to_c_u16(o) :                             \
-     sizeof(type) == 4 ? _cffi_to_c_u32(o) :                             \
-     sizeof(type) == 8 ? _cffi_to_c_u64(o) :                             \
+#define _cffi_to_c_int(o, type)                                          \
+    (sizeof(type) == 1 ? (((type)-1) > 0 ? _cffi_to_c_u8(o)              \
+                                         : _cffi_to_c_i8(o)) :           \
+     sizeof(type) == 2 ? (((type)-1) > 0 ? _cffi_to_c_u16(o)             \
+                                         : _cffi_to_c_i16(o)) :          \
+     sizeof(type) == 4 ? (((type)-1) > 0 ? _cffi_to_c_u32(o)             \
+                                         : _cffi_to_c_i32(o)) :          \
+     sizeof(type) == 8 ? (((type)-1) > 0 ? _cffi_to_c_u64(o)             \
+                                         : _cffi_to_c_i64(o)) :          \
      (Py_FatalError("unsupported size for type " #type), 0))
 
 #define _cffi_to_c_i8                                                    \
@@ -141,11 +139,13 @@ static void _cffi_init(void)
     if (c_api_object == NULL)
         return;
     if (!PyCapsule_CheckExact(c_api_object)) {
+        Py_DECREF(c_api_object);
         PyErr_SetNone(PyExc_ImportError);
         return;
     }
     memcpy(_cffi_exports, PyCapsule_GetPointer(c_api_object, "cffi"),
            _CFFI_NUM_EXPORTS * sizeof(void *));
+    Py_DECREF(c_api_object);
 }
 
 #define _cffi_type(num) ((CTypeDescrObject *)PyList_GET_ITEM(_cffi_types, num))
@@ -796,27 +796,27 @@ _cffi_f_CCCalibratePBKDF(PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple(args, "OOOOOO:CCCalibratePBKDF", &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
     return NULL;
 
-  x0 = _cffi_to_c_UNSIGNED(arg0, uint32_t);
+  x0 = _cffi_to_c_int(arg0, uint32_t);
   if (x0 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x1 = _cffi_to_c_UNSIGNED(arg1, size_t);
+  x1 = _cffi_to_c_int(arg1, size_t);
   if (x1 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x3 = _cffi_to_c_UNSIGNED(arg3, uint32_t);
+  x3 = _cffi_to_c_int(arg3, uint32_t);
   if (x3 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x4 = _cffi_to_c_UNSIGNED(arg4, size_t);
+  x4 = _cffi_to_c_int(arg4, size_t);
   if (x4 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x5 = _cffi_to_c_UNSIGNED(arg5, uint32_t);
+  x5 = _cffi_to_c_int(arg5, uint32_t);
   if (x5 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -826,7 +826,7 @@ _cffi_f_CCCalibratePBKDF(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_UNSIGNED(result, unsigned int);
+  return _cffi_from_c_int(result, unsigned int);
 }
 
 static PyObject *
@@ -852,15 +852,15 @@ _cffi_f_CCCryptorCreate(PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple(args, "OOOOOOO:CCCryptorCreate", &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
     return NULL;
 
-  x0 = _cffi_to_c_UNSIGNED(arg0, uint32_t);
+  x0 = _cffi_to_c_int(arg0, uint32_t);
   if (x0 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x1 = _cffi_to_c_UNSIGNED(arg1, uint32_t);
+  x1 = _cffi_to_c_int(arg1, uint32_t);
   if (x1 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, uint32_t);
+  x2 = _cffi_to_c_int(arg2, uint32_t);
   if (x2 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -875,7 +875,7 @@ _cffi_f_CCCryptorCreate(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x4 = _cffi_to_c_UNSIGNED(arg4, size_t);
+  x4 = _cffi_to_c_int(arg4, size_t);
   if (x4 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -907,7 +907,7 @@ _cffi_f_CCCryptorCreate(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -943,19 +943,19 @@ _cffi_f_CCCryptorCreateWithMode(PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple(args, "OOOOOOOOOOOO:CCCryptorCreateWithMode", &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &arg9, &arg10, &arg11))
     return NULL;
 
-  x0 = _cffi_to_c_UNSIGNED(arg0, uint32_t);
+  x0 = _cffi_to_c_int(arg0, uint32_t);
   if (x0 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x1 = _cffi_to_c_UNSIGNED(arg1, uint32_t);
+  x1 = _cffi_to_c_int(arg1, uint32_t);
   if (x1 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, uint32_t);
+  x2 = _cffi_to_c_int(arg2, uint32_t);
   if (x2 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x3 = _cffi_to_c_UNSIGNED(arg3, uint32_t);
+  x3 = _cffi_to_c_int(arg3, uint32_t);
   if (x3 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -981,7 +981,7 @@ _cffi_f_CCCryptorCreateWithMode(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x6 = _cffi_to_c_UNSIGNED(arg6, size_t);
+  x6 = _cffi_to_c_int(arg6, size_t);
   if (x6 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -996,15 +996,15 @@ _cffi_f_CCCryptorCreateWithMode(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x8 = _cffi_to_c_UNSIGNED(arg8, size_t);
+  x8 = _cffi_to_c_int(arg8, size_t);
   if (x8 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x9 = _cffi_to_c_SIGNED(arg9, int);
+  x9 = _cffi_to_c_int(arg9, int);
   if (x9 == (int)-1 && PyErr_Occurred())
     return NULL;
 
-  x10 = _cffi_to_c_UNSIGNED(arg10, uint32_t);
+  x10 = _cffi_to_c_int(arg10, uint32_t);
   if (x10 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1025,7 +1025,7 @@ _cffi_f_CCCryptorCreateWithMode(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1067,7 +1067,7 @@ _cffi_f_CCCryptorFinal(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1088,7 +1088,7 @@ _cffi_f_CCCryptorFinal(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1128,7 +1128,7 @@ _cffi_f_CCCryptorGCMAddAAD(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1138,7 +1138,7 @@ _cffi_f_CCCryptorGCMAddAAD(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1178,7 +1178,7 @@ _cffi_f_CCCryptorGCMAddIV(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1188,7 +1188,7 @@ _cffi_f_CCCryptorGCMAddIV(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1230,7 +1230,7 @@ _cffi_f_CCCryptorGCMDecrypt(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1251,7 +1251,7 @@ _cffi_f_CCCryptorGCMDecrypt(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1293,7 +1293,7 @@ _cffi_f_CCCryptorGCMEncrypt(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1314,7 +1314,7 @@ _cffi_f_CCCryptorGCMEncrypt(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1371,7 +1371,7 @@ _cffi_f_CCCryptorGCMFinal(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1398,7 +1398,7 @@ _cffi_f_CCCryptorGCMReset(PyObject *self, PyObject *arg0)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1425,7 +1425,7 @@ _cffi_f_CCCryptorRelease(PyObject *self, PyObject *arg0)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1471,7 +1471,7 @@ _cffi_f_CCCryptorUpdate(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1486,7 +1486,7 @@ _cffi_f_CCCryptorUpdate(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x4 = _cffi_to_c_UNSIGNED(arg4, size_t);
+  x4 = _cffi_to_c_int(arg4, size_t);
   if (x4 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1507,7 +1507,7 @@ _cffi_f_CCCryptorUpdate(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int32_t);
+  return _cffi_from_c_int(result, int32_t);
 }
 
 static PyObject *
@@ -1581,7 +1581,7 @@ _cffi_f_CCHmacInit(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x1 = _cffi_to_c_UNSIGNED(arg1, uint32_t);
+  x1 = _cffi_to_c_int(arg1, uint32_t);
   if (x1 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1596,7 +1596,7 @@ _cffi_f_CCHmacInit(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x3 = _cffi_to_c_UNSIGNED(arg3, size_t);
+  x3 = _cffi_to_c_int(arg3, size_t);
   if (x3 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1646,7 +1646,7 @@ _cffi_f_CCHmacUpdate(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1687,7 +1687,7 @@ _cffi_f_CCKeyDerivationPBKDF(PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple(args, "OOOOOOOOO:CCKeyDerivationPBKDF", &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8))
     return NULL;
 
-  x0 = _cffi_to_c_UNSIGNED(arg0, uint32_t);
+  x0 = _cffi_to_c_int(arg0, uint32_t);
   if (x0 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1702,7 +1702,7 @@ _cffi_f_CCKeyDerivationPBKDF(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, size_t);
+  x2 = _cffi_to_c_int(arg2, size_t);
   if (x2 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1717,15 +1717,15 @@ _cffi_f_CCKeyDerivationPBKDF(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x4 = _cffi_to_c_UNSIGNED(arg4, size_t);
+  x4 = _cffi_to_c_int(arg4, size_t);
   if (x4 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x5 = _cffi_to_c_UNSIGNED(arg5, uint32_t);
+  x5 = _cffi_to_c_int(arg5, uint32_t);
   if (x5 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
-  x6 = _cffi_to_c_UNSIGNED(arg6, unsigned int);
+  x6 = _cffi_to_c_int(arg6, unsigned int);
   if (x6 == (unsigned int)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1740,7 +1740,7 @@ _cffi_f_CCKeyDerivationPBKDF(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x8 = _cffi_to_c_UNSIGNED(arg8, size_t);
+  x8 = _cffi_to_c_int(arg8, size_t);
   if (x8 == (size_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1750,7 +1750,7 @@ _cffi_f_CCKeyDerivationPBKDF(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -1794,7 +1794,7 @@ _cffi_f_CC_MD5_Final(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -1821,7 +1821,7 @@ _cffi_f_CC_MD5_Init(PyObject *self, PyObject *arg0)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -1861,7 +1861,7 @@ _cffi_f_CC_MD5_Update(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, uint32_t);
+  x2 = _cffi_to_c_int(arg2, uint32_t);
   if (x2 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1871,7 +1871,7 @@ _cffi_f_CC_MD5_Update(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -1915,7 +1915,7 @@ _cffi_f_CC_SHA1_Final(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -1942,7 +1942,7 @@ _cffi_f_CC_SHA1_Init(PyObject *self, PyObject *arg0)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -1982,7 +1982,7 @@ _cffi_f_CC_SHA1_Update(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, uint32_t);
+  x2 = _cffi_to_c_int(arg2, uint32_t);
   if (x2 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -1992,7 +1992,7 @@ _cffi_f_CC_SHA1_Update(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2036,7 +2036,7 @@ _cffi_f_CC_SHA224_Final(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2063,7 +2063,7 @@ _cffi_f_CC_SHA224_Init(PyObject *self, PyObject *arg0)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2103,7 +2103,7 @@ _cffi_f_CC_SHA224_Update(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, uint32_t);
+  x2 = _cffi_to_c_int(arg2, uint32_t);
   if (x2 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -2113,7 +2113,7 @@ _cffi_f_CC_SHA224_Update(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2157,7 +2157,7 @@ _cffi_f_CC_SHA256_Final(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2184,7 +2184,7 @@ _cffi_f_CC_SHA256_Init(PyObject *self, PyObject *arg0)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2224,7 +2224,7 @@ _cffi_f_CC_SHA256_Update(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, uint32_t);
+  x2 = _cffi_to_c_int(arg2, uint32_t);
   if (x2 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -2234,7 +2234,7 @@ _cffi_f_CC_SHA256_Update(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2278,7 +2278,7 @@ _cffi_f_CC_SHA384_Final(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2305,7 +2305,7 @@ _cffi_f_CC_SHA384_Init(PyObject *self, PyObject *arg0)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2345,7 +2345,7 @@ _cffi_f_CC_SHA384_Update(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, uint32_t);
+  x2 = _cffi_to_c_int(arg2, uint32_t);
   if (x2 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -2355,7 +2355,7 @@ _cffi_f_CC_SHA384_Update(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2399,7 +2399,7 @@ _cffi_f_CC_SHA512_Final(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2426,7 +2426,7 @@ _cffi_f_CC_SHA512_Init(PyObject *self, PyObject *arg0)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static PyObject *
@@ -2466,7 +2466,7 @@ _cffi_f_CC_SHA512_Update(PyObject *self, PyObject *args)
       return NULL;
   }
 
-  x2 = _cffi_to_c_UNSIGNED(arg2, uint32_t);
+  x2 = _cffi_to_c_int(arg2, uint32_t);
   if (x2 == (uint32_t)-1 && PyErr_Occurred())
     return NULL;
 
@@ -2476,7 +2476,7 @@ _cffi_f_CC_SHA512_Update(PyObject *self, PyObject *args)
   _cffi_save_errno();
   Py_END_ALLOW_THREADS
 
-  return _cffi_from_c_SIGNED(result, int);
+  return _cffi_from_c_int(result, int);
 }
 
 static void _cffi_check_struct_CC_MD5state_st(struct CC_MD5state_st *p)
